@@ -35,6 +35,9 @@
 #include "servicelog.h"
 #include "indicator.h"
 #include "lp_util.h"
+#include "utils.h"
+
+static int rtas_event;
 
 /* FRU callout priority as defined in PAPR+
  *
@@ -172,8 +175,10 @@ service_event_supported(struct sl_event *event)
 			return 0;
 		}
 		break;
-	case SL_TYPE_BMC:
 	case SL_TYPE_RTAS:
+		rtas_event = 1;
+		break;
+	case SL_TYPE_BMC:
 	case SL_TYPE_BASIC:
 	default:
 		return 0;
@@ -300,6 +305,7 @@ build_callout_loc_code(struct sl_event *event, struct loc_code *list,
 			}
 
 			strncpy(fru->location, location, LOCATION_LENGTH - 1);
+			fru->location[LOCATION_LENGTH - 1] = '\0';
 		}
 	}
 
@@ -343,6 +349,8 @@ event_fru_callout(struct sl_callout *callouts, struct loc_code *list,
 		/* get FRUs nearest fault indicator */
 		strncpy(location, callout->location, LOCATION_LENGTH);
 		location[LOCATION_LENGTH - 1] = '\0';
+		trim_trail_space(location);
+
 		loc_led = get_fru_indicator(list, location, &truncated);
 		if (!loc_led) { /* No indicator found for the given loc code */
 			*attn_on = 1;
@@ -442,14 +450,20 @@ parse_service_event(int event_id)
 	attn_loc = &list[0];
 
 	if (operating_mode == LED_MODE_LIGHT_PATH) {
-		if (event->callouts)
+		if (event->callouts) {
 			/* Run over FRU callout priority in order and
 			 * enable fault indicator
 			 */
-			for (i = 0; FRU_CALLOUT_PRIORITY[i]; i++)
+			if (!rtas_event) {
+				for (i = 0; FRU_CALLOUT_PRIORITY[i]; i++)
+					rc = event_fru_callout(event->callouts, list,
+							FRU_CALLOUT_PRIORITY[i],
+							&attn_on);
+			} else {
 				rc = event_fru_callout(event->callouts, list,
-						       FRU_CALLOUT_PRIORITY[i],
-						       &attn_on);
+						'H', &attn_on);
+			}
+		}
 		else {
 			/* No callout list, enable check log indicator */
 			indicator_log_write("Empty callout list");
@@ -652,11 +666,11 @@ UI_help(MENU *my_menu)
 	}
 
 	if (!strcmp(desc, "ident"))
-		mvwprintw(my_text_win, 0, 0, help1);
+		mvwprintw(my_text_win, 0, 0, "%s", help1);
 	else if (!strcmp(desc, "attn"))
-		mvwprintw(my_text_win, 0, 0, help2);
+		mvwprintw(my_text_win, 0, 0, "%s", help2);
 	else
-		mvwprintw(my_text_win, 0, 0, help3);
+		mvwprintw(my_text_win, 0, 0, "%s", help3);
 
 	wrefresh(my_text_win);
 
@@ -736,7 +750,7 @@ UI_commit(WINDOW *my_menu_win, MENU *my_menu,
 	ITEM	**items;
 	ITEM	*cur;
 	WINDOW	*my_help_win;
-	struct	loc_code *loc;
+	struct	loc_code *loc = NULL;
 
 	getmaxyx(stdscr, y, x);
 	my_help_win = newwin(6, 55, y - 7, (x - 55)/2);
@@ -789,6 +803,10 @@ UI_commit(WINDOW *my_menu_win, MENU *my_menu,
 				if (rc)
 					err = 1;
 			} else {
+				if (!loc) {
+					err = 1;
+					continue;
+				}
 				rc = set_indicator_state(LED_TYPE_IDENT, loc,
 							 ident ? LED_STATE_OFF : LED_STATE_ON);
 
@@ -1122,8 +1140,8 @@ startup_window(void)
 	}
 
 	mvprintw(0, 0, "SERVICE INDICATORS VERSION %s", VERSION);
-	mvprintw(2, 0, msg1);
-	mvprintw(5, 0, msg2);
+	mvprintw(2, 0, "%s", msg1);
+	mvprintw(5, 0, "%s", msg2);
 	mvprintw(9, 0, "Press the F3 key to exit or press Enter to continue.");
 
 	while ((c = getch()) !=  KEY_F(3)) {
@@ -1179,9 +1197,11 @@ main(int argc, char *argv[])
 	int	event_id = 0;
 	int	repair_id = 0;
 	char	*next_char;
+	int platformonly = false;
 
 	program_name = argv[0];
-	if (probe_indicator() != 0) {
+	if ((platform_initialize() != 0) ||
+		(probe_indicator(platformonly) != 0)) {
 		fprintf(stderr,
 			"%s is not supported on this platform\n", argv[0]);
 		return 0;
