@@ -19,16 +19,21 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#define _GNU_SOURCE
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <librtas.h>
 #include <librtasevent.h>
+#include <syslog.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
+#include <sys/statvfs.h>
 #include "utils.h"
 #include "rtas_errd.h"
 
@@ -38,6 +43,7 @@
 #define SCANLOG_DUMP_FILE	"/proc/ppc64/scan-log-dump"
 #define SCANLOG_DUMP_EXISTS	"/proc/device-tree/chosen/ibm,scan-log-data"
 #define SYSID_FILE		"/proc/device-tree/system-id"
+#define SYSID_VENDOR_FILE	"/proc/device-tree/ibm,vendor-system-id"
 #define SCANLOG_MODULE		"scanlog"
 #define MODPROBE_PROGRAM	"/sbin/modprobe"
 
@@ -57,21 +63,26 @@ get_machine_serial()
 {
 	FILE *fp;
 	char buf[20] = {0,}, *ret = NULL;
+	char *filename = SYSID_VENDOR_FILE;
 
 	/*
 	 * Odds of SYSID_FILE, open failing is almost none.
 	 * But, better to catch the odds.
 	 */
-	fp = fopen(SYSID_FILE, "r");
+	fp = fopen(filename, "r");
 	if (!fp) {
-		log_msg(NULL, "%s: Failed to open %s, %s",
-				__func__, SYSID_FILE, strerror(errno));
-		return ret;
+		filename = SYSID_FILE;
+		fp = fopen(filename, "r");
+		if (!fp) {
+			log_msg(NULL, "%s: Failed to open %s, %s", __func__,
+					filename, strerror(errno));
+			return ret;
+		}
 	}
 
 	if (fgets(buf, sizeof(buf), fp) == NULL) {
 		log_msg(NULL, "%s: Reading file %s failed, %s",
-				__func__, SYSID_FILE, strerror(errno));
+				__func__, filename, strerror(errno));
 	} else {
 		ret = strdup(buf + 4);
 		if (!ret) {
@@ -163,8 +174,8 @@ void
 check_scanlog_dump(void)
 {
 	int rc, in = -1, out = -1, bytes;
-	char *temp;
-	char scanlog_filename[80], scanlog_filename_bak[85];
+	char *temp = NULL;
+	char *scanlog_file = NULL, *scanlog_file_bak = NULL;
 	char *dump_buf = NULL;
 
 	rc = load_scanlog_module(1);
@@ -173,27 +184,33 @@ check_scanlog_dump(void)
 
 	/* A dump exists;  copy it off to the filesystem */
 	if ((temp = get_machine_serial()) != 0) {
-		sprintf(scanlog_filename, "%sscanoutlog.%s",
-			d_cfg.scanlog_dump_path, temp);
+		if (asprintf(&scanlog_file, "%sscanoutlog.%s",
+			d_cfg.scanlog_dump_path, temp) < 0) {
+			free(temp);
+			goto scanlog_out;
+		}
 		free(temp);
 	}
 	else {
-		sprintf(scanlog_filename, "%sscanoutlog.NOSERIAL",
-			d_cfg.scanlog_dump_path);
+		if (asprintf(&scanlog_file, "%sscanoutlog.NOSERIAL",
+			     d_cfg.scanlog_dump_path) < 0)
+			goto scanlog_out;
 	}
 
-	if (!access(scanlog_filename, F_OK)) {
+	if (!access(scanlog_file, F_OK)) {
 		/* A scanlog dump already exists on the filesystem;
 		 * rename it with a .bak extension. 
 		 */
-		sprintf(scanlog_filename_bak, "%s.bak", scanlog_filename);
-		rc = rename(scanlog_filename, scanlog_filename_bak);
+		if (asprintf(&scanlog_file_bak, "%s.bak", scanlog_file) < 0)
+			goto scanlog_out;
+		rc = rename(scanlog_file, scanlog_file_bak);
 		if (rc) {
 			log_msg(NULL, "Could not rename %s, an existing "
 				"scanlog dump will be copied over by a new "
-				"dump, %s", scanlog_filename, strerror(errno));
+				"dump, %s", scanlog_file, strerror(errno));
 		}
 	}
+	free(scanlog_file_bak);
 
 	in = open(SCANLOG_DUMP_FILE, O_RDONLY);
 	if (in <= 0) {
@@ -202,11 +219,11 @@ check_scanlog_dump(void)
 		goto scanlog_error;
 	}
 
-	out = open(scanlog_filename, O_WRONLY | O_CREAT | O_TRUNC,
+	out = open(scanlog_file, O_WRONLY | O_CREAT | O_TRUNC,
 			S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 	if (out <= 0) {
 		log_msg(NULL, "Could not open %s for writing, %s",
-			scanlog_filename, strerror(errno));
+			scanlog_file, strerror(errno));
 		goto scanlog_error;
 	}
 
@@ -221,7 +238,7 @@ check_scanlog_dump(void)
 		rc = write(out, dump_buf, bytes);
 		if (rc < bytes) {
 			log_msg(NULL, "Could not write to %s, %s",
-				scanlog_filename, strerror(errno));
+				scanlog_file, strerror(errno));
 			goto scanlog_error;
 		}
 	}
@@ -230,7 +247,7 @@ check_scanlog_dump(void)
 		log_msg(NULL, "Could not read from %s, %s", SCANLOG_DUMP_FILE,
 			strerror(errno));
 
-	temp = scanlog_filename + strlen(d_cfg.scanlog_dump_path);
+	temp = scanlog_file + strlen(d_cfg.scanlog_dump_path);
 	scanlog = strdup(temp);
 	if (scanlog == NULL) {
 		log_msg(NULL, "Could not allocate space for scanlog filename, "
@@ -239,6 +256,7 @@ check_scanlog_dump(void)
 	} /* strdup(scanlog) */
 
 scanlog_out:
+	free(scanlog_file);
 	if (in != -1)
 		close(in);
 	if (out != -1)
@@ -274,9 +292,11 @@ void
 check_platform_dump(struct event *event)
 {
 	struct rtas_dump_scn *dump_scn;
+	struct statvfs vfs;
 	uint64_t dump_tag;
+	uint64_t dump_size;
 	char	filename[DUMP_MAX_FNAME_LEN + 20], *pos;
-	char	pathname[DUMP_MAX_FNAME_LEN + 40];
+	char	*pathname = NULL;
 	FILE	*f;
 	int	rc, bytes;
 	char    *system_args[3] = {NULL, };     /* execv arguments      */
@@ -296,11 +316,34 @@ check_platform_dump(struct event *event)
 		return;
 	}
 
-	/* Retrieve the dump */
+	/* Retrieve the dump tag */
 	dump_tag = dump_scn->id;
 	dump_tag |= ((uint64_t)dump_scn->v6hdr.subtype << 32);
 	dbg("Dump ID: 0x%016LX", dump_tag);
 
+	if (statvfs(d_cfg.platform_dump_path, &vfs) == -1) {
+		log_msg(event, "statvfs() failed on %s: %s",
+				d_cfg.platform_dump_path, strerror(errno));
+		return;
+	}
+
+	/* Retrieve the size of the platform dump */
+	dump_size = dump_scn->size_hi;
+	dump_size <<= 32;
+	dump_size |= dump_scn->size_lo;
+
+	/* Check if there is sufficient space in the file system to store the dump */
+	if (vfs.f_bavail * vfs.f_frsize < dump_size) {
+		syslog(LOG_ERR, "Insufficient space in %s to store platform dump for dump ID: "
+				"0x%016lX (required: %lu bytes, available: %lu bytes)",
+				d_cfg.platform_dump_path, dump_tag, dump_size,
+				(vfs.f_bavail * vfs.f_frsize));
+		syslog(LOG_ERR, "After clearing space, run 'extract_platdump "
+				"0x%016lX'.\n", dump_tag);
+		return;
+	}
+
+	/* Retrieve the dump */
 	snprintf(tmp_sys_arg, 60, "0x%016LX", (long long unsigned int)dump_tag);
 	system_args[0] = EXTRACT_PLATDUMP_CMD;
 	system_args[1] = tmp_sys_arg;
@@ -315,7 +358,12 @@ check_platform_dump(struct event *event)
 		setup_sigchld_handler();
 		return;
 	}
-	fgets(filename, DUMP_MAX_FNAME_LEN + 20, f);
+	if (!fgets(filename, DUMP_MAX_FNAME_LEN + 20, f)) {
+		dbg("Failed to collect filename info");
+		spclose(f, cpid);
+		setup_sigchld_handler();
+		return;
+	}
 	rc = spclose(f, cpid);
 
 	setup_sigchld_handler();
@@ -339,10 +387,14 @@ check_platform_dump(struct event *event)
 		bytes += (4 - (bytes % 4));
 	dump_scn->id_len = bytes;
 
-	snprintf(pathname, DUMP_MAX_FNAME_LEN + 40, "%s/%s",
-		 d_cfg.platform_dump_path, filename);
+	if (asprintf(&pathname, "%s/%s",
+		 d_cfg.platform_dump_path, filename) < 0) {
+		dbg("%s: Failed to allocate memory", __func__);
+		return;
+	}
 	platform_log_write("Platform Dump Notification\n");
 	platform_log_write("    Dump Location: %s\n", pathname);
+	free(pathname);
 
 	return;
 }
