@@ -3,6 +3,20 @@
  * @brief File Manipulation routines for files used by rtas_errd
  *
  * Copyright (C) 2004 IBM Corporation
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -11,16 +25,17 @@
 #include <unistd.h>
 #include <errno.h>
 #include <time.h>
-#include <stdarg.h> 
+#include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
+#include <libgen.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <sys/wait.h>
 #include "rtas_errd.h"
 
 char *platform_log = "/var/log/platform";
-int platform_log_fd = -1;	
+int platform_log_fd = -1;
 
 /**
  * @var proc_error_log1 
@@ -72,7 +87,7 @@ char *epow_status_file = "/var/log/epow_status";
  * @brief buffer to hold scanlog dump path
  * 
  * This is a buffer that is allocated and filled when rtas_errd is 
- * intially exec()'ed via check_scanlog_dump(). The buffer will contain 
+ * initially exec()'ed via check_scanlog_dump(). The buffer will contain
  * the path to a scanlog dump and is reported with the first RTAS event 
  * we receive from the kernel.
  */
@@ -126,21 +141,21 @@ event_dump(struct event *event)
 	char *bufp = (char *)event->rtas_hdr;
 	int len = event->length;
 
- 	/* Print 16 bytes/line in hex, with a space after every 4 bytes */
- 	for (i = 0; i < len; i++) {
- 		if ((i % 16) == 0) 
+	/* Print 16 bytes/line in hex, with a space after every 4 bytes */
+	for (i = 0; i < len; i++) {
+		if ((i % 16) == 0)
 			printf("RTAS %d:", i/16);
- 
- 		if ((i % 4) == 0)
+
+		if ((i % 4) == 0)
 			printf(" ");
- 
+
 		printf("%02x", bufp[i]);
- 
- 		if ((i % 16) == 15)
+
+		if ((i % 16) == 15)
 			printf("\n");
- 	}
- 	if ((i % 16) != 0)
- 		printf("\n");
+	}
+	if ((i % 16) != 0)
+		printf("\n");
 }
 
 /** 
@@ -169,14 +184,14 @@ setup_rtas_event_scenario(void)
 
 	fd = open(scenario_file, O_RDONLY);
 	if (fd == -1) {
-		log_msg(NULL, "Could not open scenario file %s, %s", 
+		log_msg(NULL, "Could not open scenario file %s, %s",
 			scenario_file, strerror(errno));
 		return -1;
 	}
 
 	if ((fstat(fd, &sbuf)) < 0) {
-		log_msg(NULL, "Could not get status of scenario file %s, %s", 
-			scenario_file, strerror(errno));	
+		log_msg(NULL, "Could not get status of scenario file %s, %s",
+			scenario_file, strerror(errno));
 		close(fd);
 		return -1;
 	}
@@ -190,16 +205,23 @@ setup_rtas_event_scenario(void)
 	}
 
 	len = read(fd, scenario_buf, sbuf.st_size);
+	if (len == -1) {
+		log_msg(NULL, "Could not read from scenario file %s, %s",
+			scenario_file, strerror(errno));
+		close(fd);
+		return -1;
+	}
+
 	close(fd);
 
 	/* Now, convert all '\n' and EOF chars to '\0' */
-	for (i = 0; i < sbuf.st_size; i++) {
+	for (i = 0; i < len; i++) {
 		if (scenario_buf[i] == '\n') {
 			scenario_buf[i] = '\0';
 			scenario_count++;
 		}
 	}
-	
+
 	/* Allocate the scenario array */
 	scenario_files = malloc(scenario_count * sizeof(char *));
 	if (scenario_files == NULL) {
@@ -250,29 +272,46 @@ init_files(void)
 		fprintf(stderr, "Could not open rtas_errd log file \"%s\".\n",
 			rtas_errd_log);
 
-	
+
 	/* Next, open the /proc file */
 #ifdef DEBUG
 	rc = setup_rtas_event_scenario();
-	if (rc) 
+	if (rc)
 		return rc;
 #endif
 	proc_error_log_fd = open(proc_error_log1, O_RDONLY);
-	if (proc_error_log_fd <= 0) 
+	if (proc_error_log_fd < 0)
 		proc_error_log_fd = open(proc_error_log2, O_RDONLY);
-	
-	if (proc_error_log_fd <= 0) {
-		log_msg(NULL, "Could not open error log file at either %s or "
-			"%s, %s\nThe rtas_errd daemon cannot continue and will "
-			"exit", proc_error_log1, proc_error_log2, 
-			strerror(errno));
+
+	if (proc_error_log_fd < 0) {
+
+		if (strlen(proc_error_log1) <= 2048) {
+			log_msg(NULL, "Could not open error log file at either %s or "
+					"%s, %s\nThe rtas_errd daemon cannot continue and will "
+					"exit", proc_error_log1, proc_error_log2,
+					strerror(errno));
+		} else {
+			/* If the filename is too long, truncate it */
+			char truncated_filename[2048];
+
+			strncpy(truncated_filename, proc_error_log1,
+					sizeof(truncated_filename) - 1);
+			truncated_filename[sizeof(truncated_filename) - 1] = '\0';
+
+			log_msg(NULL, "Truncating the filename since it is too long "
+					"(%d characters)", strlen(proc_error_log1));
+			log_msg(NULL, "Could not open error log file at either %s or "
+					"%s, %s\nThe rtas_errd daemon cannot continue and will "
+					"exit", truncated_filename, proc_error_log2,
+					strerror(errno));
+		}
 		return -1;
 	}
 
 	/* Next, open /var/log/platform */
 	platform_log_fd = open(platform_log, O_RDWR | O_SYNC | O_CREAT,
 			       S_IRUSR | S_IWUSR | S_IRGRP /*0640*/);
-	if (platform_log_fd <= 0) {
+	if (platform_log_fd < 0) {
 		log_msg(NULL, "Could not open log file %s, %s\nThe daemon "
 			"cannot continue and will exit", platform_log,
 			strerror(errno));
@@ -329,7 +368,7 @@ int
 read_proc_error_log(char *buf, int buflen)
 {
 	int len = 0;
-	
+
 #ifdef DEBUG
 	/* If we are reading in a test file with an ascii RTAS event,
 	 * we need to convert it to binary.  proc_error_log2 should
@@ -348,11 +387,11 @@ read_proc_error_log(char *buf, int buflen)
 				proc_error_log1, strerror(errno));
 			return -1;
 		}
-				
+
 		tf_mmap = mmap(0, tf_sbuf.st_size, PROT_READ, MAP_PRIVATE,
 			       proc_error_log_fd, 0);
 		if (tf_mmap == MAP_FAILED) {
-			log_msg(NULL, "Cannot map test file %s, %s", 
+			log_msg(NULL, "Cannot map test file %s, %s",
 				proc_error_log1, strerror(errno));
 			return -1;
 		}
@@ -387,7 +426,7 @@ read_proc_error_log(char *buf, int buflen)
 				/* go past the extra space */
 				j++;
 			}
-				
+
 			if ((data[j] == '\n') || (data[j] == ' ')) {
 				j++;
 				continue;
@@ -396,10 +435,13 @@ read_proc_error_log(char *buf, int buflen)
 			str[0] = data[j];
 			str[1] = data[j + 1];
 			j += 2;
-			sscanf(str, "%02x", &ch);
+			if (sscanf(str, "%02x", &ch) != 1)
+				continue;
+
 			buf[k++] = ch;
 			if (k >= buflen) { /* Buffer overflow */
 				log_msg(NULL, "Invalid test file");
+				munmap(tf_mmap, tf_sbuf.st_size);
 				return -1;
 			}
 		}
@@ -412,21 +454,21 @@ read_proc_error_log(char *buf, int buflen)
 				close(proc_error_log_fd);
 				tmp = scenario_files[scenario_index++];
 				proc_error_log_fd = open(tmp, O_RDONLY);
-				if (proc_error_log_fd <= 0)
+				if (proc_error_log_fd < 0)
 					log_msg(NULL, "Could not open scenario "
 						"file %s, %s", tmp,
 						strerror(errno));
-			} 
+			}
 			else {
 				testing_finished = 1;
 			}
-		} 
+		}
 		else
 			testing_finished = 1;
 
 		if (tf_mmap)
 			munmap(tf_mmap, tf_sbuf.st_size);
-	} 
+	}
 	else
 #endif
 		len = read(proc_error_log_fd, buf, buflen);
@@ -458,7 +500,7 @@ reformat_msg(char *msg)
 	}
 
 	memset(buf, 0, RTAS_ERROR_LOG_MAX);
-	
+
 	/* first copy the msg into our buffer */
 	memcpy(buf, msg, len);
 
@@ -504,38 +546,41 @@ reformat_msg(char *msg)
 /**
  * _log_msg
  * @brief The real routine to write messages to rtas_errd_log
- * 
+ *
  * This is a common routine for formatting messages that go to the
  * /var/log/rtas_errd.log file.  Users should pass in a reference to
  * the rtas_event structure if this message is directly related a rtas
  * event and a formatted message a la printf() style.  Please make sure
  * that the message passed in does not have any ending punctuation or
- * ends with a newline.  It sould also not have any internal newlines.
+ * ends with a newline.  It should also not have any internal newlines.
  *
  * This routine will do several things to the message before printing
  * it out;
- * - Add a timestamp 
+ * - Add a timestamp
  * - If a rtas_event reference is passed in, a sequenbce number is added
  * - If errno is set, the results of perror are added.
- * - The entire message is then formatted to fit in 80 cols. 
- * 
+ * - The entire message is then formatted to fit in 80 cols.
+ *
  * @param event reference to event
  * @param fmt formatted string a la printf()
  * @param ... additional args a la printf()
  */
-static void 
+static void
 _log_msg(struct event *event, const char *fmt, va_list ap)
 {
 	struct stat sbuf;
 	char	buf[RTAS_ERROR_LOG_MAX];
 	int	len = 0, rc;
+	char	*dir_name;
+        char	*rtas_errd_log_c; /* Both rtas_errd_log/rtas_errd_log0 are
+				     global, so make a copy. */
 
 	if (rtas_errd_log_fd == -1) {
 		dbg("rtas_errd log file is not available");
-		
+
 		vsprintf(buf, fmt, ap);
 		_dbg(buf);
-	
+
 		return;
 	}
 
@@ -558,7 +603,7 @@ _log_msg(struct event *event, const char *fmt, va_list ap)
 
 	/* Add the actual message */
 	len += vsprintf(buf + len, fmt, ap);
-	
+
 	/* Add ending punctuation */
 	len += sprintf(buf + len, ".");
 
@@ -570,38 +615,80 @@ _log_msg(struct event *event, const char *fmt, va_list ap)
 	rc = write(rtas_errd_log_fd, buf, len);
 	if (rc == -1)
 		dbg("rtas_errd: write to rtas_errd log failed");
-	
+
 	/* After each write check to see if we need to to do log rotation */
 	rc = fstat(rtas_errd_log_fd, &sbuf);
 	if (rc == -1) {
-		dbg("rtas_errd: Cannot stat rtas_errd log file " 
+		dbg("rtas_errd: Cannot stat rtas_errd log file "
 		    "to rotate logs");
 		return;
 	}
 
 	if (sbuf.st_size >= RTAS_ERRD_LOGSZ) {
-		char	cmd_buf[1024];
-		int	rc;
-		int8_t	status;
+		int rc;			/* return code */
+		struct stat log_file;	/* file stat   */
+		int dir_fd;		/* fsync       */
 
 		close(rtas_errd_log_fd);
 
-		memset(cmd_buf, 0, 1024);
-		sprintf(cmd_buf, "rm -f %s; mv %s %s", rtas_errd_log0,
-			rtas_errd_log, rtas_errd_log0); 
-		rc = system(cmd_buf);
+		/*
+		 * Remove log file /var/log/rtas_errd.log0
+		 */
+		rc = stat(rtas_errd_log0, &log_file);
+		if (rc == 0) {
+			rc = unlink(rtas_errd_log0);
+			if (rc == -1) {
+				log_msg(NULL, "An error occurred during the "
+					"rotation of the rtas_errd logs:\n "
+					"cmd = rm %s\nexit status: %d)",
+					rtas_errd_log0, errno);
+			}
+		} /* stat */
+
+		/*
+		 * rotate /var/log/rtas_errd.log to /var/log/rtas_errd.log0
+		 */
+		rc = rename(rtas_errd_log, rtas_errd_log0);
 		if (rc == -1) {
-			status = WEXITSTATUS(rc);
-			log_msg(NULL, "An error occured during the rotation of "
-				"the rtas_errd logs:\n cmd = %s\nexit status: "
-				"%d)", cmd_buf, status);
+			log_msg(NULL, "An error occurred during rotation of "
+				"rtas_errd logs (logger alive):\n cmd = rename"
+				" %s %s\nexit status: %d)", rtas_errd_log,
+				rtas_errd_log0, errno);
 		}
 
-		rtas_errd_log_fd = open(rtas_errd_log, 
+		/*
+		 * fsync /var/log/ to be  on safer side.
+		 */
+		rtas_errd_log_c = strdup(rtas_errd_log);
+		if (!rtas_errd_log_c) {
+			log_msg(NULL, "%s, Memory allocation failed, %s",
+					__func__, strerror(errno));
+			rtas_errd_log_fd = -1;
+			dbg("Could not re-open %s", rtas_errd_log);
+			return;
+
+		}
+
+		dir_name = dirname(rtas_errd_log_c);
+		dir_fd = open(dir_name, O_RDONLY|O_DIRECTORY);
+		if (dir_fd == -1) {
+			log_msg(NULL, "Could not open directory %s (%s:%d)",
+					dir_name, strerror(errno), errno);
+		} else {
+			rc = fsync(dir_fd);
+			if (rc == -1)
+				log_msg(NULL, "fsync failed, on %s\nexit status:"
+						" %d", dir_name, errno);
+			close(dir_fd);
+		}
+
+		rtas_errd_log_fd = open(rtas_errd_log,
 					O_RDWR | O_CREAT | O_TRUNC,
 					S_IRUSR | S_IWUSR | S_IRGRP);
 		if (rtas_errd_log_fd == -1)
 			dbg("Could not re-open %s", rtas_errd_log);
+
+		free(rtas_errd_log_c);
 	}
 
 	return;
@@ -652,7 +739,7 @@ log_msg(struct event *event, char *fmt, ...)
  * @param fmt format string a la printf()
  * @param ... args a la printf()
  */
-void 
+void
 _dbg(const char *fmt, ...)
 {
 	va_list ap;
@@ -684,7 +771,7 @@ _dbg(const char *fmt, ...)
  * @param event pointer to the struct event to print
  * @return 0 on success, !0 on failure
  */
-int 
+int
 print_rtas_event(struct event *event)
 {
 	char	*out_buf;
@@ -692,7 +779,7 @@ print_rtas_event(struct event *event)
 	int	i, rc, nlines;
 
 	/* Determine the length of the log */
- 	len = event->length;
+	len = event->length;
 
 	/* Some RTAS events (i.e. shortened epow events) only consist
 	 * of the rtas header and an extended length of zero.  For these
@@ -720,18 +807,17 @@ print_rtas_event(struct event *event)
 	buf_size = (len * 2) + (nlines * 0x10) + (2 * 0x40);
 	/* event_dump(event); */
 
-	out_buf = malloc(buf_size); 
+	out_buf = calloc(1, buf_size);
 	if (out_buf == NULL) {
 		log_msg(NULL, "Could not allocate buffer to print RTAS event "
-			"%d, %s.  The event will not copied to %s", 
+			"%d, %s.  The event will not copied to %s",
 			event->seq_num, strerror(errno),
 			platform_log);
 		return -1;
 	}
-	memset(out_buf, 0, buf_size);
-	       
-	offset = sprintf(out_buf, 
-			 "RTAS: %d -------- RTAS event begin --------\n", 
+
+	offset = sprintf(out_buf,
+			 "RTAS: %d -------- RTAS event begin --------\n",
 			 event->seq_num);
 
 	if (event->flags & RE_SCANLOG_AVAIL) {
@@ -739,26 +825,26 @@ print_rtas_event(struct event *event)
 		free(scanlog);
 		scanlog = NULL;
 	}
-	
- 	/* Print 16 bytes/line in hex, with a space after every 4 bytes */
- 	for (i = 0; i < len; i++) {
- 		if ((i % 16) == 0)
- 			offset += sprintf(out_buf + offset, "RTAS %d:", i/16);
- 
- 		if ((i % 4) == 0)
- 			offset += sprintf(out_buf + offset, " ");
- 
- 		offset += sprintf(out_buf + offset, "%02x", 
-				  event->event_buf[i]); 
- 
- 		if ((i % 16) == 15) 
+
+	/* Print 16 bytes/line in hex, with a space after every 4 bytes */
+	for (i = 0; i < len; i++) {
+		if ((i % 16) == 0)
+			offset += sprintf(out_buf + offset, "RTAS %d:", i/16);
+
+		if ((i % 4) == 0)
+			offset += sprintf(out_buf + offset, " ");
+
+		offset += sprintf(out_buf + offset, "%02x",
+				  event->event_buf[i]);
+
+		if ((i % 16) == 15)
 			offset += sprintf(out_buf + offset, "\n");
- 	}
- 	if ((i % 16) != 0)
- 		offset += sprintf(out_buf + offset, "\n");
-	 
- 	offset += sprintf(out_buf + offset,
-			  "RTAS: %d -------- RTAS event end ----------\n", 
+	}
+	if ((i % 16) != 0)
+		offset += sprintf(out_buf + offset, "\n");
+
+	offset += sprintf(out_buf + offset,
+			  "RTAS: %d -------- RTAS event end ----------\n",
 			  event->seq_num);
 
 	dbg("Writing RTAS event %d to %s", event->seq_num, platform_log);
@@ -766,8 +852,8 @@ print_rtas_event(struct event *event)
 	rc = write(platform_log_fd, out_buf, offset);
 	if (rc != offset) {
 		log_msg(NULL, "Writing RTAS event %d to %s failed."
-			"expected to write %d, only wrote %d. %s", 
-			event->seq_num, platform_log, offset, rc, 
+			"expected to write %d, only wrote %d. %s",
+			event->seq_num, platform_log, offset, rc,
 			strerror(errno));
 	}
 
@@ -797,7 +883,7 @@ platform_log_write(char *fmt, ...)
 	len = sprintf(buf, "%s ", "ppc64-diag:");
 
 	va_start(ap, fmt);
-	len += vsprintf(buf + len, fmt, ap);
+	len += vsnprintf(buf + len, (1024 - len), fmt, ap);
 	va_end(ap);
 
 	rc = write(platform_log_fd, &buf, len);
